'use strict';

exports.__esModule = true;

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

exports.createTemplate = createTemplate;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodashAssign = require('lodash.assign');

var _lodashAssign2 = _interopRequireDefault(_lodashAssign);

var defaultStyles = {
  menu: {
    position: 'absolute',
    top: '100%',
    padding: 0,
    width: '200px'
  },
  menuHidden: {
    display: 'none',
    zIndex: '-1'
  },
  menuVisible: {
    display: 'block',
    zIndex: 500
  },
  container: {
    position: 'relative',
    display: 'inline-block'
  },
  alignRight: {
    right: 0
  },
  alignLeft: {
    left: 0
  },
  alignCenter: {
    left: '50%',
    transform: 'translateX(-50%)'
  },
  arrow: {
    width: 0,
    height: 0,
    border: '5px solid transparent',
    borderTopWidth: 0,
    marginTop: '5px',
    padding: 0,
    position: 'absolute',
    top: '10px',
    left: '50%',
    marginLeft: '-5px',
    borderBottomColor: '#999'
  },
  list: {
    padding: 0,
    paddingTop: '5px',
    paddingBottom: '5px',
    margin: 0,
    backgroundColor: 'white',
    border: '1px solid #ddd',
    outline: 'none'
  }
};

function createTemplate() {
  var styles = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  return _lodashAssign2['default']({}, defaultStyles, styles);
}

var Dropdown = (function (_React$Component) {
  _inherits(Dropdown, _React$Component);

  _createClass(Dropdown, null, [{
    key: 'propTypes',
    value: {
      showing: _react.PropTypes.bool.isRequired,
      onRequestHide: _react.PropTypes.func.isRequired,
      onRequestShow: _react.PropTypes.func.isRequired,
      activeIndex: _react.PropTypes.number,
      trigger: _react.PropTypes.node.isRequired,
      autoHideOnClick: _react.PropTypes.bool,
      arrow: _react.PropTypes.bool,
      align: _react.PropTypes.oneOf(['center', 'left', 'right']),
      uniqueId: _react.PropTypes.string.isRequired,
      template: _react.PropTypes.object
    },
    enumerable: true
  }, {
    key: 'defaultProps',
    value: {
      align: 'center',
      arrow: false,
      autoHideOnClick: true,
      template: defaultStyles
    },
    enumerable: true
  }]);

  function Dropdown() {
    _classCallCheck(this, Dropdown);

    _React$Component.call(this);
    this._onClick = this.onClick.bind(this);
  }

  Dropdown.prototype.render = function render() {
    var _props = this.props;
    var showing = _props.showing;
    var arrow = _props.arrow;
    var uniqueId = _props.uniqueId;
    var align = _props.align;
    var trigger = _props.trigger;
    var template = _props.template;

    var triggerClone = _react2['default'].cloneElement(trigger, {
      onClick: this.onTriggerClick.bind(this),
      'aria-haspopup': true,
      role: 'button',
      id: uniqueId,
      ref: 'trigger'
    });

    var positionStyle = template.alignCenter;
    if (align === 'left') {
      positionStyle = template.alignLeft;
    } else if (align === 'right') {
      positionStyle = template.alignRight;
    }

    var menuStyle = _lodashAssign2['default']({}, template.menu, showing ? template.menuVisible : template.menuHidden, positionStyle);

    return _react2['default'].createElement(
      'div',
      { className: 'Menu', style: template.container },
      triggerClone,
      _react2['default'].createElement(
        'div',
        { className: 'Menu__container', style: menuStyle },
        showing && _react2['default'].createElement('div', { tabIndex: '0' }),
        arrow && _react2['default'].createElement('div', { style: template.arrow }),
        _react2['default'].createElement(
          'ul',
          { className: 'Menu__list', onKeyDown: this.onKeyDown.bind(this),
            ref: 'menu',
            role: 'menu',
            tabIndex: '-1',
            'aria-hidden': this.props.showing ? 'false' : 'true',
            'aria-describedby': uniqueId,
            style: template.list },
          _react2['default'].Children.map(this.props.children, this.renderChild.bind(this))
        ),
        showing && _react2['default'].createElement('div', { tabIndex: '0' })
      )
    );
  };

  Dropdown.prototype.onKeyDown = function onKeyDown(e) {
    if (e.key === 'Escape') {
      this.hideDropdown();
    }
  };

  Dropdown.prototype.onTriggerClick = function onTriggerClick(e) {
    e.preventDefault();
    e.stopPropagation();
    if (this.props.showing) {
      this.hideDropdown();
    } else {
      this.showDropdown();
    }
  };

  Dropdown.prototype.showDropdown = function showDropdown() {
    var _this = this;

    if (this.props.showing) return;
    this.props.onRequestShow();
    setTimeout(function () {
      document.addEventListener('click', _this._onClick);
      document.addEventListener('touch', _this._onClick);
      _this.refs.menu.focus();
    }, 0);
  };

  Dropdown.prototype.hideDropdown = function hideDropdown() {
    this.props.onRequestHide();
    document.removeEventListener('click', this._onClick);
    document.removeEventListener('touch', this._onClick);
    this.refs.trigger.focus();
  };

  Dropdown.prototype.onClick = function onClick(e) {
    var _this2 = this;

    var el = this.refs.menu;

    if (this.props.autoHideOnClick) {
      setTimeout(function () {
        _this2.hideDropdown();
      }, 0);
      return;
    }

    if (!el.contains(e.target)) {
      this.hideDropdown();
      return;
    }
  };

  Dropdown.prototype.renderChild = function renderChild(child) {
    // add ability to select item from parent element,
    // for example, using keyboard nav
    return _react2['default'].cloneElement(child, {});
  };

  return Dropdown;
})(_react2['default'].Component);

exports['default'] = Dropdown;